// based on hyper-john music playing functionality
// https://github.com/cwright017/hyper-john

const AUDIO_SRC = require('./cringe.js');

exports.middleware = (store) => (next) => (action) => {
  if ('SESSION_ADD_DATA' === action.type) {
    const { data } = action;
    if (/(cringe: command not found)|(command not found: cringe)/.test(data)) {
      store.dispatch({
        type: 'CRINGE_CALL'
      });
    } else {
      next(action);
    }
  } else {
    next(action);
  }
};

exports.reduceUI = (state, action) => {
  switch (action.type) {
    case 'CRINGE_CALL':
      return state.set('cringeCall', !state.cringeCall);
  }
  return state;
};

exports.mapTermsState = (state, map) => {
  return Object.assign(map, {
    cringeCall: state.ui.cringeCall
  });
};

const passProps = (uid, parentProps, props) => {
  return Object.assign(props, {
    cringeCall: parentProps.cringeCall
  });
}

module.exports.decorateTerm = (Term, {React, notify}) => {
    return class extends React.Component {
        constructor (props, context) {
            super(props, context)
            this._onTerminal = this._onTerminal.bind(this);
            this._audio = new Audio(AUDIO_SRC);
            this._audio.onended = this._onAudioEnded.bind(this);
            this._audio.onplay = this._onAudioPlay.bind(this);
        }

        _resetAudio() {
            this._audio.pause();
            this._audio.currentTime = 0;
            clearInterval(this._audioPlaying);
        }

        _onAudioEnded () {
            this._resetAudio();            
        }

        _onTerminal (term) {
            this._div = term.div_;
            this._window = term.document_.defaultView;
        
            this._audio.play();

            if (this.props.onTerminal) {
                this.props.onTerminal(term);
            }
        }

        componentWillReceiveProps (next) {
            if (next.cringeCall && !this.props.cringeCall) {
                notify('CRINGE!!!');
            }
        }

        componentWillUnmount () {            
            this._resetAudio();
        }

    render () {
      return React.createElement(Term, Object.assign({}, this.props, {
        onTerminal: this._onTerminal
      }))
    }
  }
}
